@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017
+;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -34,22 +34,26 @@
   #:use-module (artanis third-party csv)
   #:use-module (artanis third-party json)
   #:use-module (artanis server)
+  #:use-module (artanis cache)
   #:use-module (artanis websocket)
+  #:use-module (artanis lpc)
   #:use-module (artanis version)
   #:use-module (web server)
   #:use-module (srfi srfi-1)
   #:use-module (ice-9 format)
-  #:re-export (;; page module
-               params
+  #:re-export (;; env
+               http-status
+
+               ;; page module
                response-emit
                throw-auth-needed
                tpl->html
                redirect-to
                tpl->response
                reject-method
-               response-error-emit
                run-after-request!
                run-before-response!
+               run-when-sigint!
                emit-response-with-file
                static-page-emitter
 
@@ -58,6 +62,7 @@
                post
                put
                patch
+               page-options
                page-delete
                :sql-mapping
                :str
@@ -70,11 +75,14 @@
                :cookies-update!
                :cookies-remove!
                :cookies-setattr!
+               :cookies-check
+               :cookies-value
                :mime
                :auth
                :session
                :from-post
                :websocket
+               :lpc
 
                ;; db module
                DB-open
@@ -120,12 +128,15 @@
                rc-mtime rc-mtime!
                rc-cookie rc-cookie!
                rc-set-cookie rc-set-cookie!
+               rc-oht rc-oht!
                rc-conn rc-conn!
                new-route-context
                route-context?
                get-header
                get-from-qstr
-               
+               get-referer
+               params
+
                ;; csv
                make-csv-reader
                csv->xml
@@ -141,20 +152,17 @@
                json-string->scm
                json-parser?
                json-parser-port
-               json
-               
+
                ;; upload
                mfd-simple-dump
                make-mfd-dumper
                content-type-is-mfd?
-               parse-mfd-body
                mfd
                call-with-mfd-data
                find-mfd
                make-mfd
                mfd?
                is-mfds?
-               mfds-count
                mfd-dispos
                mfd-name
                mfd-filename
@@ -165,6 +173,17 @@
                store-uploaded-files
                upload-files-to
 
+               ;; cache
+               try-to-get-page-from-cache
+               cache-this-page
+               clear-content-cache
+
+               ;; websocket
+               named-pipe-subscribe
+
+               ;; server
+               schedule-task
+
                ;; version
                artanis-version)
   #:export (result-ref
@@ -197,48 +216,53 @@
   (default-route-init statics cache-statics? exclude)
   (init-hook)
   (init-config)
-  (init-server-core)
+  (init-lpc)
   (check-invalid-config)
-  (sigaction SIGPIPE SIG_IGN)
+  (sigaction SIGPIPE SIG_IGN) ; surpass SIGPIPE signal since we want to handle EPIPE by self
+  (sigaction SIGINT (lambda (i)
+                      (run-when-sigint-hook)
+                      (format (artanis-current-output)
+                              "~%Fare you well, your server is cold.~%")
+                      (quit)))
   (set! is-init-server-run? #t))
 
 (define* (form-tag #:key (controller #f) (action #f) (method #f)
                    (class #f) (tag-class #f) (tag-id #f) (form-method "get"))
   (lambda tags
     (call-with-output-string
-     (lambda (port)
-       (format port "<form accept-charset='~a'" (get-conf '(server charset)))
-       (format port " action='~a'"
-               (call-with-output-string
-                (lambda (port2)
-                  (display "/" port2)
-                  (and controller (format port2 "~a/" controller))
-                  (and action (format port2 "~a?" action))
-                  (and method (format port2 "method=~a" method))
-                  (and class (format port2 "&class=~a" class)))))
-       (format port " method='~a'" method)
-       (and tag-class (format port " class='~a'" tag-class))
-       (and tag-id (format port " id='~a'" tag-id))
-       (display ">\n" port)
-       (for-each (lambda (tag) (format port "~a~%" tag)) tags)
-       (format port "</form>~%")))))
+      (lambda (port)
+        (format port "<form accept-charset='~a'" (get-conf '(server charset)))
+        (format port " action='~a'"
+                (call-with-output-string
+                  (lambda (port2)
+                    (display "/" port2)
+                    (and controller (format port2 "~a/" controller))
+                    (and action (format port2 "~a?" action))
+                    (and method (format port2 "method=~a" method))
+                    (and class (format port2 "&class=~a" class)))))
+        (format port " method='~a'" method)
+        (and tag-class (format port " class='~a'" tag-class))
+        (and tag-id (format port " id='~a'" tag-id))
+        (display ">\n" port)
+        (for-each (lambda (tag) (format port "~a~%" tag)) tags)
+        (format port "</form>~%")))))
 
 (define (make-general-tag tag)
   (lambda attrs
     (lambda (contents)
       (call-with-output-string
-       (lambda (port)
-         (format port "<~a" tag)
-         (let lp((next attrs))
-           (cond
-            ((null? next)
-             (display ">\n" port)
-             (display contents port)
-             (newline port)
-             (format port "</~a>" tag))
-            (else
-             (format port " ~a='~a'" (keyword->symbol (car next)) (cadr next))
-             (lp (cddr attrs))))))))))
+        (lambda (port)
+          (format port "<~a" tag)
+          (let lp((next attrs))
+            (cond
+             ((null? next)
+              (display ">\n" port)
+              (display contents port)
+              (newline port)
+              (format port "</~a>" tag))
+             (else
+              (format port " ~a='~a'" (keyword->symbol (car next)) (cadr next))
+              (lp (cddr attrs))))))))))
 
 (define label-tag (make-general-tag 'label))
 (define a-tag (make-general-tag 'a))
@@ -260,7 +284,7 @@
                  "[WARN] You're not in application folder, the file motoring is disabled!~%"))))
 
 ;; Invalid use-db? must be (dbd username passwd) or #f
-(define* (run #:key (host #f) (port #f) (debug #f) (use-db? #f) (db-proto #f)
+(define* (run #:key (host #f) (port #f) (debug #f) (use-db? #f) (db-proto #f) (server #f)
               (dbd #f) (db-username #f) (db-passwd #f) (db-name #f) (db-addr #f))
   (define (->proper-body-display body)
     (cond
@@ -276,36 +300,39 @@
       (if debug
           (print-websocket-frame body)
           "<websocket-frame>"))
-     (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type!" body))))
+     (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type `~a'!" body))))
   (when (not is-init-server-run?)
     (error "Sorry, but you have to run (init-server) in the begining of you main program!"))
   (and host (conf-set! '(host addr) host))
   (and port (conf-set! '(host port) port))
+  (and server (conf-set! '(server engine) (string->symbol server)))
+  (init-server-core)
   (when debug
     (display "DEBUG: ON\n")
     (init-debug-mode))
-  (when (or use-db? (get-conf 'use-db?))
-    (conf-set! 'use-db? #t)
+  (when (or use-db? (get-conf '(db enable)))
+    (conf-set! '(db enable) #t)
     (display "User wants to use Database, initializing...\n")
     (init-database-config dbd db-username db-passwd db-name db-addr db-proto)
     (init-DB)
     (display "DB init done!\n")
     (when (eq? 'db (get-conf '(session backend)))
-          (session-init)
-          (display "Session with DB backend init done!\n")))
+      (session-init)
+      (display "Session with DB backend init done!\n")))
   (case (get-conf '(session backend))
     ((db)
-     (when (not (get-conf 'use-db?))
-           (error "Session with DB backend init failed because you didn't enable DB!")))
+     (when (not (get-conf '(db enable)))
+       (error "Session with DB backend init failed because you didn't enable DB!")))
     (else
      (session-init)
      (format #t "Session with ~:@(~a~) backend init done!~%"
              (get-conf '(session backend)))))
   (run-hook *before-run-hook*)
+  (format #t "Server core: ~a~%" (get-conf '(server engine)))
   (format #t "~a~%" (current-myhost))
   (format #t "Anytime you want to quit just try Ctrl+C, thanks!~%")
   (let ((handler (if debug
-                     (lambda (r b)
+                     (lambda (r b . _)
                        (format #t "[Request] ~a~%[Body] ~a~%" r (->proper-body-display b))
                        (server-handler r b))
                      server-handler)))
