@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2017,2018
+;;  Copyright (C) 2014,2015,2017,2018,2019
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -24,12 +24,13 @@
   #:use-module (artanis irregex)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
-  #:export (make-handler-rc
-            handler-rc?
-            handler-rc-handler
-            handler-rc-keys
-            handler-rc-oht
-            get-handler-rc
+  #:export (make-handler-context
+            handler-context?
+            handler-context-handler
+            handler-context-keys
+            handler-context-uid
+            handler-context-oht
+            get-handler-context
 
             rc-handler rc-handler!
             rc-keys rc-keys!
@@ -44,6 +45,7 @@
             rc-mtime rc-mtime!
             rc-cookie rc-cookie!
             rc-set-cookie rc-set-cookie!
+            rc-oht rc-oht!
             rc-conn rc-conn!
             rc-lpc rc-lpc!
             new-route-context
@@ -56,21 +58,27 @@
             init-rule-key-bindings!
             init-query!
             get-from-qstr
-            get-referer))
+            get-referer
+            rc-oht-ref
+            params
+            get-rule-uid))
 
-(define-record-type handler-rc
-  (make-handler-rc handler keys oht)
-  handler-rc?
-  (handler handler-rc-handler)
-  (keys handler-rc-keys)
-  (oht handler-rc-oht))
+(define-record-type handler-context
+  (make-handler-context handler keys uid oht)
+  handler-context?
+  (handler handler-context-handler)
+  (keys handler-context-keys)
+  ;; NOTE: We record an uid of each raw-rule so that we don't have to match the rule
+  ;;       redundantly.
+  (uid handler-context-uid)
+  (oht handler-context-oht))
 
-(define (get-handler-rc handler-key)
+(define (get-handler-context handler-key)
   (hash-ref *handlers-table* handler-key))
 
 (define-record-type route-context
   (make-route-context handler keys regexp request path qt method rhk bt
-                      body date cookie set-cookie conn lpc)
+                      body date cookie set-cookie oht conn lpc)
   route-context?
   (handler rc-handler rc-handler!) ; reqeust handler
   (keys rc-keys rc-keys!) ; rule keys
@@ -91,6 +99,7 @@
   (set-cookie rc-set-cookie rc-set-cookie!) ; the cookies needed to be set as response
   ;; auto DB connection doesn't need users to close it, it's auto closed when request is over.
   (lpc rc-lpc rc-lpc!) ; store lpc object for later destruction
+  (oht rc-oht rc-oht!) ; Option Handlers Table
   (conn rc-conn rc-conn!)) ; auto DB connection from pool
 
 (define (get-header rc k)
@@ -107,7 +116,7 @@
          (method (if (eq? m 'HEAD) 'GET m))
          (cookies (request-cookies request))
          (rc (make-route-context #f #f #f request path #f method #f #f
-                                 body #f cookies '() #f #f)))
+                                 body #f cookies '() #f #f #f)))
     ;; FIXME: maybe we don't need rhk? Throw it after get handler & keys
     (init-rule-handler-key! rc) ; set rule handler key
     (init-rule-handler-and-keys! rc) ; set handler and keys
@@ -134,13 +143,14 @@
 (define (init-rule-handler-and-keys! rc)
   (let* ((handler-key (rc-rhk rc))
          (hrc (if handler-key  ; get handler-keys pair
-                  (get-handler-rc handler-key)
+                  (get-handler-context handler-key)
                   (throw 'artanis-err 404 init-rule-handler-and-keys!
                          "Client ~a had visited an invalid path ~a"
                          (REASON-TEXT (remote-info (rc-req rc)))
                          (REASON-TEXT (rc-path rc))))))
-    (rc-handler! rc (handler-rc-handler hrc))
-    (rc-keys! rc (reverse (handler-rc-keys hrc)))))
+    (rc-oht! rc (handler-context-oht hrc))
+    (rc-handler! rc (handler-context-handler hrc))
+    (rc-keys! rc (reverse (handler-context-keys hrc)))))
 
 (define (init-rule-path-regexp! rc)
   (rc-re! rc (string->irregex (cdr (rc-rhk rc)))))
@@ -182,3 +192,24 @@
             #f
             referer)
         #f)))
+
+;; NOTE: oht will be #f in these situations:
+;; 1. URL is not hit
+;; 2. URL is static files
+(define (rc-oht-ref rc key)
+  (and=> (rc-oht rc) (lambda (oht) (hash-ref oht key))))
+
+;; The params will be searched in binding-list first, then search from
+;; qstr
+;; TODO: qstr should be independent from rules binding.
+(define (params rc key)
+  ((current-encoder)
+   (or (assoc-ref (rc-bt rc) key)
+       (get-from-qstr rc key))))
+
+(define (get-rule-uid rc)
+  (let ((hc (get-handler-context (rc-rhk rc))))
+    (when (not hc)
+      (throw 'artanis-err 500 get-rule-uid
+             "BUG: If handler-context is missing, then there shouldn't be a rc!"))
+    (handler-context-uid hc)))
