@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018
+;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -18,10 +18,7 @@
 ;;  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (artanis utils)
-  #:use-module (artanis crypto md5)
-  #:use-module (artanis crypto sha-1)
-  #:use-module (artanis crypto sha-2)
-  #:use-module (artanis crypto base64)
+  #:use-module (artanis security nss)
   #:use-module (artanis tpl sxml)
   #:use-module (artanis config)
   #:use-module (artanis irregex)
@@ -33,12 +30,13 @@
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module (ice-9 ftw)
-  #:use-module (srfi srfi-1)
-  #:use-module (srfi srfi-19)
+  #:use-module (ice-9 iconv)
   #:use-module (ice-9 local-eval)
   #:use-module (ice-9 receive)
   #:use-module (ice-9 q)
   #:use-module (ice-9 control)
+  #:use-module (srfi srfi-1)
+  #:use-module (srfi srfi-19)
   #:use-module (web http)
   #:use-module (web request)
   #:use-module (web response)
@@ -48,18 +46,17 @@
                           utf8->string put-bytevector
                           bytevector-u8-ref string->utf8 bytevector-length
                           make-bytevector bytevector-s32-native-ref bytevector?
-                          define-record-type record-rtd record-accessor
-                          get-string-all))
+                          define-record-type record-rtd
+                          get-string-all bitwise-ior))
   #:export (regexp-split
             hash-keys cat bv-cat get-global-time sanitize-response
-            get-local-time string->md5 unsafe-random parse-date write-date
+            get-local-time unsafe-random parse-date write-date
             get-file-ext get-global-date get-local-date
             string-substitute nfx static-filename remote-info seconds-now local-time-stamp
             make-expires export-all-from-module!
             alist->hashtable expires->time-utc local-eval-string
             time-expired? valid-method? mmap munmap get-random-from-dev
-            string->byteslist string->sha-1 string->sha-224 string->sha-256 string->sha-384
-            string->sha-512 list-slice bv-slice uni-basename detect-type-name
+            string->byteslist list-slice bv-slice uni-basename detect-type-name
             checkout-the-path make-string-template guess-mime prepare-headers
             new-stack new-queue stack-slots queue-slots stack-pop! stack-push!
             stack-top stack-empty? queue-out! queue-in! queue-head queue-tail
@@ -69,8 +66,8 @@
             keyword->string range oah->handler oah->opts string->keyword
             alist->klist alist->kblist is-hash-table-empty?
             symbol-downcase symbol-upcase normalize-column run-before-run!
-            sxml->xml-string run-after-request! run-before-response! run-when-DB-init!
-            run-when-sigint! make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
+            sxml->xml-string run-after-request! run-before-response! run-when-DB-init! run-when-sigint-hook
+            run-when-sigint! run-when-refresh! make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
             generate-kv-from-post-qstr handle-proper-owner run-after-websocket-handshake!
             generate-data-url verify-ENTRY exclude-dbd
             draw-expander remove-ext scan-app-components cache-this-route!
@@ -78,13 +75,12 @@
             handle-existing-file check-drawing-method DEBUG
             subbv->string subbv=? bv-read-line bv-read-delimited put-bv
             bv-u8-index bv-u8-index-right build-bv-lookup-table filesize
-            plist-remove gen-migrate-module-name try-to-load-migrate-cache
-            flush-to-migration-cache gen-local-conf-file with-dbd
+            plist-remove gen-migrate-module-name gen-local-conf-file with-dbd
             call-with-sigint define-box-type make-box-type unbox-type
             ::define did-not-specify-parameter colorize-string-helper
             colorize-string WARN-TEXT ERROR-TEXT REASON-TEXT
             NOTIFY-TEXT STATUS-TEXT get-trigger get-family get-addr request-path
-            response-keep-alive? request-keep-alive?
+            response-keep-alive? request-keep-alive? file->bytevector
             procedure-name->string proper-toplevel gen-content-length
             make-file-sender file-sender? file-sender-size file-sender-thunk
             get-string-all-with-detected-charset make-unstop-exception-handler
@@ -93,7 +89,8 @@
             artanis-sys-response char-predicate handle-upload is-valid-table-name?
             is-guile-compatible-server-core? positive-integer? negative-integer?
             io-exception:peer-is-shutdown? io-exception:out-of-memory?
-            out-of-system-resources? allow-long-live-connection?)
+            out-of-system-resources? allow-long-live-connection?
+            free-JS-announcement generate-rule-uid gen-cache-file)
   #:re-export (the-environment
                utf8->string
                bytevector?
@@ -119,7 +116,16 @@
                request-headers
                request-method
                request-content-length
-               request-port))
+               request-port
+
+               (nss:md5 . string->md5)
+               (nss:sha-1 . string->sha-1)
+               (nss:sha-224 . string->sha-224)
+               (nss:sha-256 . string->sha-256)
+               (nss:sha-384 . string->sha-384)
+               (nss:sha-512 . string->sha-512)
+               nss:base64-encode
+               nss:base64-decode))
 
 (define parse-date (@@ (web http) parse-date))
 (define write-date (@@ (web http) write-date))
@@ -271,8 +277,8 @@
 
 (define-syntax-rule (static-filename path)
   (if (current-toplevel)
-      (format #f "~a/~a" (current-toplevel) path)
-      (format #f "./~a" path)))
+      (format #f "~a/pub/~a" (current-toplevel) path)
+      (format #f "./pub/~a" path)))
 
 (define-syntax-rule (request-ip req)
   ;; TODO: support AF_INET6 in the future
@@ -286,10 +292,9 @@
       (or (assoc-ref (request-headers req) 'x-real-ip) "Unknown IP")
       (or (request-ip req) "Unknown IP")))
 
-(define *methods-list* '(HEAD GET POST PUT PATCH DELETE))
+(define *methods-list* '(HEAD GET POST PUT PATCH OPTIONS DELETE))
 (define (allowed-method? method)
-  ;; TODO: check allowed method from config
-  #t)
+  (memq method (get-conf '(server allowedmethods))))
 (define (valid-method? method)
   (if (and (member method *methods-list*) (allowed-method? method))
       method
@@ -319,26 +324,54 @@
 (define-public MAP_FIXED        #x10)
 (define-public MAP_ANONYMOUS    #x20)
 (define-public MAP_UNINITIALIZED 0) ;; don't support map uninitialized
+(define-public MAP_FAILED #xffffffffffffffff)
 
-(define *libc-ffi* (dynamic-link))
+;;void *mmap(void *addr, size_t length, int prot, int flags, int fd, off_t offset)
 (define %mmap
   (pointer->procedure '*
-                      (dynamic-func "mmap" *libc-ffi*)
-                      (list '* size_t int int int size_t) #:return-errno? #t))
+                      (dynamic-func "mmap" (dynamic-link))
+                      (list '* size_t int int int long)
+                      #:return-errno? #t))
+
+;;redefine mmap as
+;;(mmap fd len addr prot flags offset)
+(define* (mmap fd len #:key (addr %null-pointer) (prot (list PROT_READ PROT_WRITE)) (flags (list MAP_SHARED)) (offset 0))
+  (let ((pv (fold bitwise-ior 0 prot))
+        (fv (fold bitwise-ior 0 flags)))
+    (call-with-values
+        (lambda()
+          (%mmap addr len pv fv fd offset))
+      (lambda(ret errno)
+        (cond
+         ((not (= (pointer-address ret) MAP_FAILED)) ret)
+         (else (throw 'artanis-error mmap
+                      "Error: ~a" (strerror errno))))))))
+
 (define %munmap
   (pointer->procedure int
-                      (dynamic-func "munmap" *libc-ffi*)
-                      (list '* size_t) #:return-errno? #t))
-(define* (mmap size #:key (addr %null-pointer) (fd -1) (prot MAP_SHARED)
-               (flags PROT_READ) (offset 0) (bv? #f))
-  (let ((ret (if bv? (lambda (p) (pointer->bytevector p size)) identity)))
-    (ret (%mmap addr size prot flags fd offset))))
-(define (munmap bv/pointer size)
-  (cond
-   ((bytevector? bv/pointer)
-    (%munmap (bytevector->pointer bv/pointer size) size))
-   ((pointer? bv/pointer)
-    (%munmap bv/pointer size))))
+                      (dynamic-func "munmap" (dynamic-link))
+                      (list '* size_t)
+                      #:return-errno? #t))
+
+(define (munmap addr len)
+  (call-with-values
+      (lambda ()
+        (%munmap addr len))
+    (lambda (ret errno)
+      (cond
+       ((>= ret 0) ret)
+       (else (throw 'artanis-error 500 munmap
+                    "Error: ~a" (strerror errno)))))))
+
+(define (file->bytevector filename)
+  (let ((size (stat:size (stat filename)))
+        (port (open-input-file filename)))
+    (let ((bv (pointer->bytevector
+               (mmap (port->fdes port) size
+                     #:prot (list PROT_READ) #:flags (list MAP_SHARED))
+               size)))
+      (close port)
+      bv)))
 
 ;; FIXME: what if len is not even?
 (define (string->byteslist str step base)
@@ -350,66 +383,6 @@
       (lp (cons (string->number (substring/shared str i (+ i step)) base) ret) (1+ i)))
      (else (lp ret (1+ i))))))
 
-(define (string->md5 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-1
-                          "need string or bytevector!" str/bv)))))
-    (md5->string (md5 in))))
-
-(define (string->sha-1 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-1
-                          "need string or bytevector!" str/bv)))))
-    (sha-1->string (sha-1 in))))
-
-(define (string->sha-224 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-224
-                          "need string or bytevector!" str/bv)))))
-    (sha-224->string (sha-224 in))))
-
-(define (string->sha-256 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-256
-                          "need string or bytevector!" str/bv)))))
-    (sha-256->string (sha-256 in))))
-
-(define (string->sha-384 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-384
-                          "need string or bytevector!" str/bv)))))
-    (sha-384->string (sha-384 in))))
-
-(define (string->sha-512 str/bv)
-  (let ((in (cond
-             ((string? str/bv)
-              ((@ (rnrs) string->utf8) str/bv))
-             (((@ (rnrs) bytevector?) str/bv)
-              str/bv)
-             (else (throw 'artanis-err 500 string->sha-512
-                          "need string or bytevector!" str/bv)))))
-    (sha-512->string (sha-512 in))))
-
 (define-syntax list-slice
   (syntax-rules (:)
     ((_ ll lo : hi)
@@ -430,9 +403,10 @@
 ;; 1. (> hi (bytevector-length bv))
 ;; 2. (< lo 0) wrap reference
 (define (%bv-slice bv lo hi)
-  (let* ((len (- hi lo))
+  (let* ((len (- hi lo -1))
          (slice ((@ (rnrs) make-bytevector) len)))
-    ((@ (rnrs) bytevector-copy!) bv lo slice 0 len) slice))
+    ((@ (rnrs) bytevector-copy!) bv lo slice 0 len)
+    slice))
 
 ;; NOT SAFE %bytevector-slice for GC, need
 ;;(define (%bytevector-slice bv lo hi)
@@ -724,6 +698,9 @@
 (define (run-when-sigint-hook)
   (run-hook *when-sigint-hook*))
 
+(define (run-when-refresh! proc)
+  (add-hook! *refresh-hook* proc))
+
 ;; NOTE: For `pipeline' methodology, please read my post:
 ;; http://nalaginrut.com/archives/2014/04/25/oba-pipeline-style%21
 (define (make-pipeline . procs)
@@ -824,11 +801,10 @@
       (else (throw 'artanis-err 500 generate-data-url
                    "Invalid MIME! Should be symbol or string"
                    mime))))
-  (let ((b64 (base64-encode bv/str)))
+  (let ((b64 (nss:base64-encode bv/str)))
     (string-concatenate (list "data:" (->mime) (->crypto) (->charset) "," b64))))
 
 (define (verify-ENTRY entry)
-  (format #t "entry: ~a~%" entry)
   (cond
    ((not (file-exists? entry))
     (format (artanis-current-output) "ENTRY file is missing!~%")
@@ -851,14 +827,20 @@
   (let ((i (string-contains str ".")))
     (substring str 0 i)))
 
-(define (scan-app-components component)
-  (let ((toplevel (current-toplevel)))
-    (map (lambda (f) (string->symbol (remove-ext f)))
-         (scandir (format #f "~a/app/~a/" toplevel component)
-                  (lambda (f)
-                    (not (or (string=? f ".")
-                             (string=? f "..")
-                             (string=? f ".gitkeep"))))))))
+(define* (scan-app-components component #:optional (sym? #t))
+  (define-syntax-rule (-> x)
+    (if sym? (string->symbol x) x))
+  (let* ((toplevel (current-toplevel))
+         (cpath (format #f "~a/app/~a/" toplevel component)))
+    (cond
+     ((file-exists? cpath)
+      (map (lambda (f) (-> (remove-ext f)))
+           (scandir cpath
+                    (lambda (f)
+                      (not (or (string=? f ".")
+                               (string=? f "..")
+                               (string=? f ".gitkeep")))))))
+     (else '()))))
 
 (define (cache-this-route! url meta)
   (define (write-header port)
@@ -878,7 +860,14 @@
           (write-header port)
           (if (eof-object? rl)
               (write '() port)
-              (write (assoc-set! rl url (drop-right meta 1)) port))
+              (write
+               (assoc-set! rl url
+                           (map (lambda (e)
+                                  (if (procedure? e)
+                                      (object->string e)
+                                      e)) meta))
+               port
+               ))
           (flock port LOCK_UN))))))
 
 (define (dump-route-from-cache)
@@ -907,7 +896,7 @@
           (for-each (lambda (r)
                       (let* ((meta (cdr r))
                              (rule (assq-ref meta 'rule))
-                             (method (assq-ref meta 'method)))
+                             (method (and=> (assq-ref meta 'method) car)))
                         (format port "~2t(~a ~s)~%"
                                 (if method method 'get)
                                 (if rule rule (car r)))))
@@ -929,16 +918,16 @@
          (component (basename (dirname pp)))
          (name (car (string-split (basename pp) #\.))))
     (cond
-     ((draw:is-force?)
+     ((cmd:is-force?)
       (if (file-is-directory? path)
           (delete-directory path)
           (delete-file path)))
-     ((draw:is-skip?)
+     ((cmd:is-skip?)
       (format (artanis-current-output) "skip ~10t app/~a/~a~%" component name))
      (else
       (format (artanis-current-output)
               "~a `~a' exists! (Use --force/-f to overwrite or --skip/-s to ignore)~%"
-              (string-capitalize component) name)
+              (string-upcase component) name)
       (exit 1)))))
 
 ;; Check if all methods are valid
@@ -950,11 +939,11 @@
             lst)
   lst)
 
-(define (subbv->string bv encoding start end)
+(define (subbv->string bv encoding start len)
   (call-with-output-string
     (lambda (port)
       (set-port-encoding! port encoding)
-      (put-bytevector port bv start (- end start)))))
+      (put-bytevector port bv start len))))
 
 (define* (bv-u8-index bv u8 #:optional (time 1))
   (let ((len (bytevector-length bv)))
@@ -997,7 +986,7 @@
   (bv-read-delimited bv 10 start end))
 
 (define (put-bv port bv from to)
-  (put-bytevector port bv from (- to from 2)))
+  (put-bytevector port bv from (- to from -1)))
 
 ;; TODO: build a char occurence indexing table
 (define (build-bv-lookup-table bv)
@@ -1038,27 +1027,6 @@
    (else (throw 'artanis-err 500 gen-migrate-module-name
                 "Wrong parsing of module name, shouldn't be here!" f))))
 
-(define (try-to-load-migrate-cache name)
-  (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
-    (cond
-     ((file-exists? file) (load file))
-     (else
-      (format (artanis-current-output)
-              "[WARN] No cache for migration of `~a'~%" name)
-      (format (artanis-current-output)
-              "Run `art migrate up ~a', then try again!~%" name)))))
-
-(define (flush-to-migration-cache name fl)
-  (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
-    (when (file-exists? file) (delete-file file))
-    (call-with-output-file file
-      (lambda (port)
-        (format port "(define-~a~%" name)
-        (for-each
-         (lambda (ft) (format port "~2t~a~%" ft))
-         fl)
-        (format port "~2t)~%")))))
-
 (define (gen-local-conf-file)
   (format #f "~a/conf/artanis.conf" (current-toplevel)))
 
@@ -1115,7 +1083,7 @@
   (list (symbol-append 'make- bt) v))
 
 (define-syntax-rule (box-type-treasure t)
-  (record-accessor (record-rtd t) 0))
+  ((@ (rnrs) record-accessor) (record-rtd t) 0))
 
 (define-syntax-rule (unbox-type t)
   (let ((treasure-getter (box-type-treasure t)))
@@ -1341,10 +1309,16 @@
                                (current-toplevel) file)))
     (if (file-exists? local-syspage)
         local-syspage
-        (format #f "~a/~a" (get-conf '(server syspage path)) file))))
+        (let ((sys-syspage (format #f "~a/~a" (get-conf '(server syspage path)) file)))
+          (and (file-exists? sys-syspage)
+               sys-syspage)))))
 
-(define (syspage-show file)
-  (bv-cat (get-syspage file) #f))
+(define (syspage-show status)
+  (let* ((file (format #f "~a.html" status))
+         (syspage (get-syspage file)))
+    (if syspage
+        (bv-cat syspage #f)
+        #vu8())))
 
 ;; ENHANCE: use colored output
 (define* (artanis-log blame-who? status mime #:key (port (current-error-port))
@@ -1373,8 +1347,7 @@
 
 (define (render-sys-page blame-who? status request)
   (artanis-log blame-who? status 'text/html #:request request)
-  (let* ((filename (format #f "~a.html" status))
-         (charset (get-conf '(server charset)))
+  (let* ((charset (get-conf '(server charset)))
          (mtime (generate-modify-time (current-time)))
          (guile-compt-serv? (is-guile-compatible-server-core? (get-conf '(server engine))))
          (response
@@ -1382,9 +1355,18 @@
                           #:headers `((server . ,(get-conf '(server info)))
                                       (last-modified . ,mtime)
                                       (content-type . (text/html (charset . ,charset))))))
-         (body (if (resources-collecting?)
-                   #vu8(0) ; Don't open any file since we don't have resources now.
-                   (syspage-show filename))))
+         (body (cond
+                ((resources-collecting?)
+                 #vu8(0)) ; Don't open any file since we don't have resources now
+                ((get-syspage-handler status)
+                 => (lambda (thunk)
+                      (let ((body (thunk)))
+                        (cond
+                         ((string? body)
+                          (string->bytevector body (get-conf '(server charset))))
+                         ((bytevector? body) body)
+                         (else (syspage-show status))))))
+                (else (syspage-show status)))))
     (if guile-compt-serv?
         (values response body)
         (values response body 'exception))))
@@ -1404,9 +1386,8 @@
   (lambda (status)
     (format-status-page/client status request)))
 
-(define (exception-from-server)
-  (lambda (status)
-    (format-status-page/server status)))
+(define (exception-from-server status)
+  (format-status-page/server status))
 
 (define *rf-re* (string->irregex ".*/artanis/artanis/(.*)$"))
 (define (->reasonable-file filename)
@@ -1434,9 +1415,12 @@
          (format port "HTTP ~a~%" (STATUS-TEXT status))
          (format port "Captured in <~a>~%" (WARN-TEXT (->reasonable-file filename)))
          (when subr (format port "Threw in procedure ~a :~%"
-                            (WARN-TEXT (if (procedure? subr)
-                                           (procedure-name->string subr)
-                                           subr))))
+                            (WARN-TEXT (cond
+                                        ((procedure? subr)
+                                         (procedure-name->string subr))
+                                        ((symbol? subr)
+                                         (symbol->string subr))
+                                        (else subr)))))
          (apply format port
                 (REASON-TEXT (string-append "[REASON] " msg))
                 args)
@@ -1547,3 +1531,65 @@
 
 (define (allow-long-live-connection?)
   (> (get-conf '(server timeout)) 0))
+
+;; Story: When I released GNU Artanis-0.2.1, RMS had asked me if I can
+;;        support LibreJS for freeing Javascript code in the generated code.
+;;        I promised I will. This took almost one year since I was very
+;;        busy on developing a new product (partly use GNU Artanis, of course),
+;;        so it's delayed. Till few months ago, RMS sent mail to ask me
+;;        if I'm ready for it, I realized that the release of GNU Artanis
+;;        is much delayed.
+;;        LibreJS is a way to detect non-free JS code to help you to avoid
+;;        the non-trivial JS code for certain hidden features. Some of them
+;;        are dangerous, some are stolen your privacy, some are just don't
+;;        let you know what's going on.
+;;        GNU Artanis may generate JS code automatically, they're licensed as
+;;        free software definitely. This free-JS-announcement is used to
+;;        license all the generated JS code in the page to GPLv3.
+;;        This announcement is useful when you have LibreJS plugin in your
+;;        browser to detect the JS automatically.
+;;        Although you're free to relicense the code to whatever you prefer,
+;;        I wish you could free the code, no matter what license name it is.
+(define free-JS-announcement
+  "
+    <script>
+       /*
+        @licstart  The following is the entire license notice for the
+        JavaScript code in this page.
+
+        Copyright (C) 2014  Loic J. Duros
+
+        The JavaScript code in this page is free software: you can
+        redistribute it and/or modify it under the terms of the GNU
+        General Public License (GNU GPL) as published by the Free Software
+        Foundation, either version 3 of the License, or (at your option)
+        any later version.  The code is distributed WITHOUT ANY WARRANTY;
+        without even the implied warranty of MERCHANTABILITY or FITNESS
+        FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.
+
+        As additional permission under GNU GPL version 3 section 7, you
+        may distribute non-source (e.g., minimized or compacted) forms of
+        that code without the copy of the GNU GPL normally required by
+        section 4, provided you include this license notice and a URL
+        through which recipients can access the Corresponding Source.
+
+
+        @licend  The above is the entire license notice
+        for the JavaScript code in this page.
+        */
+    </script>
+")
+
+;; FIXME: MD5 may not be the best choice
+(define-syntax-rule (generate-rule-uid rule)
+  (string->md5 rule))
+
+(define (gen-cache-file path)
+  (define-syntax-rule (-> str)
+    (string-trim-both
+     (irregex-replace/all "[/]+" str "_")
+     (lambda (x) (memv x '(#\sp #\_)))))
+  (let ((p (-> path)))
+    (if (string-null? p)
+        (format #f "~a/tmp/cache/index.html" (current-toplevel))
+        (format #f "~a/tmp/cache/~a.html" (current-toplevel) (-> path)))))
